package com.springpo.loginservice.service.Impl;

import com.springpo.loginservice.dto.LoginOutUserRequest;
import com.springpo.loginservice.dto.LoginUserRequest;
import com.springpo.loginservice.utils.LoginMapper;
import com.springpo.loginservice.model.LoginUser;
import com.springpo.loginservice.repo.LoginRepository;
import com.springpo.loginservice.service.LoginService;
import com.springpo.loginservice.utils.ConstantsUtils;
import com.springpoc.userservice.exception.CustomException;
import com.springpoc.userservice.model.User;
import com.springpoc.userservice.utils.UserStatus;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.common.protocol.types.Field;
import org.bson.types.ObjectId;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.listener.ListenerExecutionFailedException;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;

@Service
@Slf4j
public class LoginServiceImpl implements LoginService {

    @Autowired
    LoginRepository loginRepository;
    @Autowired
    RestTemplate restTemplate;

    private static final Logger logger = LoggerFactory.getLogger(LoginServiceImpl.class);

    //   private final ReactiveKafkaConsumerTemplate<String, User> kafkaConsumerTemplate;
/*
    @Autowired
    private KafkaTemplate<String, User> kafkaTemplate;
    private KafkaTemplate<String, String> kafkaTemplate1;
*/


    @Value("${spring.kafka.consumer.topic}")
    private String kafkaTopic;

    @Value("${spring.kafka.consumer.group-id}")
    private String groupId;

    private String generatedPassword;

    @Autowired
    WebClient.Builder webClientBuilder;

/* public LoginServiceImpl(ReactiveKafkaConsumerTemplate<String, User> reactiveKafkaConsumerTemplate) {
        this.reactiveKafkaConsumerTemplate = reactiveKafkaConsumerTemplate;
    }*/

    @Override
    // @Transactional
    public void fetchUserAndSave(User user) {
        //  generatedPassword = LoginMapper.createPassword(user);
        LoginUser loginUsers = LoginMapper.mapUserToLoginUsers(user);

        loginRepository.save(loginUsers)
                .doOnSuccess(savedLoginUser -> {
                    String generatedPassword = LoginMapper.createPassword(user);
                    // LoginMapper.savePasswordInTopic(generatedPassword);
                    System.out.println("Saved loginUsers in MongoDB: " + savedLoginUser);
                })
                .thenReturn("User entry created in login table.")
                .subscribe();

    }

    @KafkaListener(topics = "${spring.kafka.consumer.topic}", groupId = "${spring.kafka.consumer.group-id}", containerFactory = "userKafkaListenerFactory")
    private void consumeUser(User user) {
        log.info("inside consumer" + user);
        try {

            fetchUserAndSave(user);
            // Some code that may throw the exception
        } catch (ListenerExecutionFailedException e) {
            Throwable cause = e.getCause();
            System.out.println("Root cause: " + cause.getMessage());
        }

    }


    @Override
    public Flux<LoginUser> fetchAllUsersAndSave(Flux<User> users) {
        return users.flatMap(user -> {
            LoginUser loginUser = LoginMapper.mapUserToLoginUsers(user);
            return loginRepository.save(loginUser);
        });
    }

    public Mono<String> validateOtpAndGeneratePass(LoginUserRequest loginUserRequest) {
        String otp = loginUserRequest.getOtp();
        String email = loginUserRequest.getEmail();
        ObjectId Id = loginUserRequest.getId();
        log.info("otp :" + loginUserRequest.getOtp());
        log.info("EMAIL : " + Id);
        return webClientBuilder.build()
                .get()
                .uri(ConstantsUtils.userUrl, email)
                .retrieve()
                .bodyToMono(User.class)
                .flatMap(user -> {
                    // Check if there's a login entity for the given email
                    return loginRepository.findByEmail(email)
                            .flatMap(loginUser -> {
                                // Check OTP and update password if OTP is valid
                                if (loginUser.getOtp() != null && loginUser.getOtp().equalsIgnoreCase(ConstantsUtils.otp)) {
                                    String generatedPassword = LoginMapper.createPassword(user); // Generate secure password
                                    loginUser.setPassword(generatedPassword); // Set generated password
                                    loginUser.setOtp(null);
                                    loginUser.setLoginFlag(true);
                                    loginUser.setUserStatus(UserStatus.ACTIVE.getUserStatus());
                                    loginUser.setLogoutFlag(false);
                                    return loginRepository.save(loginUser)
                                            .thenReturn(ConstantsUtils.GeneratePassWord); // Save and return password generation success message
                                } else {
                                    return Mono.error(new CustomException(ConstantsUtils.InValidOtp)); // Throw error for invalid OTP
                                }
                            })
                            .switchIfEmpty(Mono.error(new CustomException("User not found"))); // Throw error if no login entity found
                }).doOnError(t -> log.error(ConstantsUtils.InValidOtp))
                .switchIfEmpty(Mono.error(new CustomException("User not exists")));

    }

    public Mono<ResponseEntity<User>> authenticatePassword(LoginUserRequest loginUserRequest) {
        String otp = loginUserRequest.getOtp();
        String email = loginUserRequest.getEmail();

        ObjectId Id = loginUserRequest.getId();
        String password = loginUserRequest.getPassword();
        return webClientBuilder.build()
                .get()
                .uri(ConstantsUtils.userUrl, email)
                .retrieve()
                .bodyToMono(User.class)
                .flatMap(user -> {
                    // Fetch the user by email from the repository
                    return loginRepository.findByEmail(email)
                            .flatMap(loginUser -> {
                                // Check if the OTP matches the predefined OTP
                                if (!loginUser.getPassword().isEmpty() && loginUser.getPassword().equalsIgnoreCase(password)) {
                                    // Set flags and update password here
                                    loginUser.setId(Id);
                                    loginUser.setLoginFlag(Boolean.TRUE);
                                    loginUser.setLogoutFlag(Boolean.FALSE);
                                    loginUser.setUserStatus(UserStatus.ACTIVE.getUserStatus());
                                    // Generate a secure password and update other flags if needed
                                    // For now, I'm assuming you have the logic to update password securely
                                    // Save the updated user in the repository
                                    return loginRepository.save(loginUser)
                                            .thenReturn(ResponseEntity.ok(user));
                                } else {
                                    // Password doesn't match, return an error
                                    return Mono.error(new CustomException(ConstantsUtils.invalidCredentials));
                                }
                            })
                            .switchIfEmpty(Mono.error(new CustomException(ConstantsUtils.unableCreateLogin)));
                });
    }


    @Override
    public Mono<String> deleteByEmailId(String email) {
        return loginRepository.deleteByEmail(email);
    }

    @Override
    public Mono<String> logoutUser(LoginOutUserRequest loginOutUserRequest) {
        ObjectId Id = loginOutUserRequest.getId();
        String email = loginOutUserRequest.getEmail();
        String password = loginOutUserRequest.getPassword();
        return loginRepository.findByEmail(email)
                .flatMap(loginUser -> {
                    // Check if the OTP matches the predefined OTP
                    if (!loginUser.getPassword().isEmpty() && loginUser.getPassword().equalsIgnoreCase(password)
                            && loginUser.getLoginFlag() == true ) {
                        // Set flags and update password here
                        loginUser.setId(Id);
                        loginUser.setLoginFlag(Boolean.FALSE);
                        loginUser.setLogoutFlag(Boolean.TRUE);
                        loginUser.setUserStatus(UserStatus.INACTIVE.getUserStatus());
                        // Generate a secure password and update other flags if needed
                        // For now, I'm assuming you have the logic to update password securely
                        // Save the updated user in the repository
                        return   loginRepository.save(loginUser)
                                 .thenReturn("Logout done successfully.");
                    } else {
                        // Password doesn't match, return an error
                        return Mono.error(new CustomException(ConstantsUtils.invalidCredentials));
                    }
                });

    }
}




