package com.springpoc.userservice.service.impl;

import com.springpoc.userservice.dto.UserDTO;

import com.springpoc.userservice.exception.CustomException;
import com.springpoc.userservice.model.User;
import com.springpoc.userservice.repo.UserRepository;
import com.springpoc.userservice.service.UserService;
import com.springpoc.userservice.utils.ConstantsUtils;
import com.springpoc.userservice.utils.Roles;

import com.springpoc.userservice.utils.UsersMapper;
import jakarta.transaction.Transactional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.SendResult;
import org.springframework.stereotype.Service;

import org.springframework.web.client.RestTemplate;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.concurrent.CompletableFuture;

@Service
public class UserServiceImpl implements UserService {
    Roles roles;
    private Boolean createUserFlag = false;
    @Autowired
    UserRepository userRepository;
    private final RestTemplate restTemplate;
    private final Logger log = LoggerFactory.getLogger(UserServiceImpl.class);
    //private final ReactiveKafkaProducerTemplate<String, User> reactiveKafkaProducerTemplate;

    @Autowired
    private KafkaTemplate<String, User> kafkaTemplate;

    @Value("${spring.kafka.producer.topic}")
    private String kafkaTopic;


    public UserServiceImpl(RestTemplate restTemplate) {
        this.restTemplate = restTemplate;

        // this.reactiveKafkaProducerTemplate = reactiveKafkaProducerTemplate;
        //  this.kafkaTopic = kafkaTopic;
        //  this.kafkaTopic = kafkaTopic1;
    }

    @Override
    public Mono<ResponseEntity<UserDTO>> createUser(Mono<User> userMono) {
        return userMono
                .filter(user -> (user.getCreatedBy().equalsIgnoreCase(roles.ADMIN.name())
                        && user.getUserType().equalsIgnoreCase(roles.RCADMIN.name())) ||
                        (user.getCreatedBy().equalsIgnoreCase(roles.RCADMIN.name())
                                && user.getUserType().equalsIgnoreCase(roles.MERCHANT.name())))
                .switchIfEmpty(Mono.error(new IllegalAccessException(ConstantsUtils.notAuthrozedToCreate)))
                .flatMap(this::notifyLoginServiceToSaveUser)
                .flatMap(userRepository::save)
                .map(UsersMapper::mapToUserDTO)
                .map(ResponseEntity::ok);
    }


    @Override
    public Flux<User> createAllUsers(Flux<User> userFlux) {
        return userFlux
                .filter(user -> (user.getCreatedBy().equalsIgnoreCase(roles.ADMIN.name())
                        && user.getUserType().equalsIgnoreCase(roles.RCADMIN.name())) ||
                        (user.getCreatedBy().equalsIgnoreCase(roles.RCADMIN.name())
                                && user.getUserType().equalsIgnoreCase(roles.MERCHANT.name())))
                .switchIfEmpty(Mono.error(new IllegalAccessException(ConstantsUtils.notAuthrozedToCreate)))
                .flatMap(user -> notifyLoginServiceToSaveUser(user))
                .collectList()
                //.flatMapMany(Flux::fromIterable)
                .flatMapMany(userRepository::saveAll);

    }



   /* private Mono<User> notifyLoginServiceToSaveUser(User user) {
        HttpEntity<User> requestEntity = new HttpEntity<>(user);
        // Make the exchange
        ResponseEntity<String> response = restTemplate.exchange(ConstantsUtils.saveUrl, HttpMethod.POST, requestEntity, String.class);
        // Check the response status
        if (response.getStatusCode().is2xxSuccessful()) {
            return Mono.just(user);  // Return the original UserDTO
        } else {
            return Mono.error(new RuntimeException(ConstantsUtils.failToNotify));
        }
    }*/

    @Override
    public Mono<ResponseEntity<User>> findByEmail(String userId) {
        return userRepository.findByEmail(userId)
                .switchIfEmpty(Mono.error(new CustomException(ConstantsUtils.userNotExists)))
                .map(ResponseEntity::ok);
    }

    @Override
    public Mono<ResponseEntity<User>> updateUser(Mono<User> userMono) {
        return userMono
                .flatMap(user -> userRepository.findById(user.getId())
                        .flatMap(existingUser -> {
                            // Update fields from the new user object
                            existingUser.setId(user.getId());
                            existingUser.setFirstName(user.getFirstName());
                            existingUser.setLastName(user.getLastName());
                            existingUser.setPhNo(user.getPhNo());
                            existingUser.setEmail(user.getEmail());
                            existingUser.setAddress(user.getAddress());
                            existingUser.setUserType(user.getUserType());
                            existingUser.setUserStatus(user.getUserStatus());
                            existingUser.setUpdatedBy(user.getUpdatedBy());
                            existingUser.setUpdatedTime(user.getUpdatedTime());
                            // Save the updated user document
                            return userRepository.save(existingUser)
                                    .flatMap(existingUser1 -> notifyLoginServiceToSaveUser(existingUser1));
                        }).
                        map(ResponseEntity::ok)
                        .switchIfEmpty(Mono.error(new CustomException(ConstantsUtils.userNotExists))));
    }

    @Transactional
    @Override
    public Mono<String> deleteByEmailId(String email) {
        return userRepository.findByEmail(email)

                .flatMap(user -> {
                    // Delete user and related login entries
                    return userRepository.deleteByEmail(email)
                            .flatMap(email1 -> notifyLoginServiceToDelete(email1))
                            //   .then(loginRepository.deleteByEmail(email))
                            .then(Mono.just("User and login entries deleted successfully"));
                })
                .switchIfEmpty(Mono.error(new CustomException(ConstantsUtils.userNotExists)));
    }

    private Mono<String> notifyLoginServiceToDelete(String email) {
        HttpEntity<String> requestEntity = new HttpEntity<>(email);
        // Make the exchange
        ResponseEntity<String> response = this.restTemplate.exchange(ConstantsUtils.deleteUrl, HttpMethod.DELETE, requestEntity, String.class);
        // Check the response status
        if (response.getStatusCode().is2xxSuccessful()) {
            return Mono.just(ConstantsUtils.userDeletedSuccessfully);  // Return the original UserDTO
        } else {
            return Mono.error(new RuntimeException(ConstantsUtils.failToNotify));
        }
    }
    public Mono<User> notifyLoginServiceToSaveUser(User user) {
        log.info("send to topic={}, {}={},", kafkaTopic);
        CompletableFuture<SendResult<String, User>> completableFuture = kafkaTemplate.send(kafkaTopic, user);
        return Mono.fromFuture(completableFuture)
                .map(sendResult -> sendResult.getProducerRecord().value());
    }
}
